// Illustration of single dynamic dispatch.  Method calls work but
// compile time type matching is static

// Series of classes each with a proclaim method; invoking the method
// dynamically dispatches
class Animal{ 
  public void proclaim(String s){ 
    System.out.println(s); 
  }
}
class Mouse extends Animal {
  public void proclaim(String s){
    super.proclaim("squeak (Mice can't talk)");
  }
}
class Snake extends Animal {
  public void proclaim(String s){
    System.out.println(s+"ssss..");
  }
}
class Duck extends Animal {
  public void proclaim(String s){
    System.out.println("quack");
  }
}
class Mascot extends Duck {
  public void proclaim(String s){
    super.proclaim(s);
    System.out.println("Aflack!");
  }
}

public class SingleDispatch{ 
  // Series of identify methods which must be matched at *compile*
  // time. No dynamic dispatch is performed for these methods.
  public static void identify(Animal x) { System.out.println("I'm an animal"); }
  public static void identify(Mouse x)  { System.out.println("I'm a mouse");   }
  public static void identify(Snake x)  { System.out.println("I'm a snake");   }
  public static void identify(Duck x)   { System.out.println("I'm a duck");    }
  public static void identify(Mascot x) { System.out.println("I'm mascot");    }

  public static void main(String args[]){
    Animal animals[] = new Animal[]{
      new Animal(),
      new Mouse(),
      new Snake(),
      new Duck(),
      new Mascot()
    };
    for(Animal a : animals){
      a.proclaim("Hello");      // Expected
      identify(a);              // Unexpected
      System.out.println();
    }
  }
}
